from functions import *

import argparse
import csv

parser = argparse.ArgumentParser(description='')
parser.add_argument("-s", "--static", help='Extract static metrics (default: false)',
                    action='store_true', default=False)
args = parser.parse_args()

result_file = os.path.join(result_directory, "results.csv")
output_files_names = os.path.join(result_directory, "output_files.csv")

static_metrics = {
    "Number of IPTAs": "IPTA",
    "Number of clocks": "X",
    "Has invariants?": "Inv.?",
    "Has clocks with rate <>1?": "neq 1 clocks?",
    "L/U subclass": "L/U?",
    "Has silent actions?": "Sil.?",
    "Is strongly deterministic?": "stDet.?",
    "Number of parameters": "P",
    "Number of discrete variables": "discr. Var",
    "Number of actions": "Act",
    "Total number of locations": "L",
    "Average locations per IPTA": "avg(L)",
    "Total number of transitions": "T",
    "Average transitions per IPTA": "avg(T)",
}
semantic_metrics = {
    "Time": "time",
    "Number of states": "states",
    "Number of attempts": "number_attempts",
    "Number of merges": "number_merges",
    "Merging time": "merging_time",
    "Reconstruction time": "reconstruction_time"
}


def extract_static_metrics(file):
    def extract(metric):
        return extract_value(file, metric)

    dict = {}
    for metric, key in static_metrics.items():
        dict[key] = extract(metric)
    return dict


def extract_semantic_metrics(file):
    def time_to_number(text):
        return text.replace("seconds", "").replace("second", "").replace(" ", "").replace(".", ",")

    dict = {}

    # Time
    if "time limit" in extract_value(file, "Termination"):
        dict["time"] = "TO"
    else:
        dict["time"] = time_to_number(extract_value(file, "Total computation time"))

    # Number of merges
    dict["number_merges"] = extract_value(file, "StateSpace.merges")
    dict["number_attempts"] = extract_value(file, "StateSpace.merging attempts")

    # Number of states
    dict["states"] = extract_value(file, "Number of states")
    dict["merging_time"] = extract_value(file, "StateSpace.Merge time")
    
    
    # Time in merging
    dict["merging_time"] = time_to_number(extract_value(file, "StateSpace.Merge time"))
    dict["reconstruction_time"] = time_to_number(extract_value(file, "StateSpace.Merge (reconstruct state space)"))

    return dict


def read_files(output_files=output_files_names):
    f_results = open(result_file, "w")
    fieldnames = ["Model", "Property", ]
    if args.static:
        fieldnames = fieldnames + list(static_metrics.values())
    for semantic_metric in semantic_metrics.keys():
        for exec in executions_to_do.keys():
            fieldnames.append("{} {}".format(semantic_metric, exec))
    writer = csv.DictWriter(f_results, fieldnames=fieldnames, delimiter=csv_sep)
    writer.writeheader()

    f_res_files = open(output_files, "r")

    reader = csv.DictReader(f_res_files, delimiter=csv_sep)
    for reading in reader:
        row = {}
        row["Model"] = reading["Model"]
        row["Property"] = reading["Property"]
        try:
            # static
            if args.static:
                static_values = extract_static_metrics(reading[list(executions_to_do.keys())[0]])
                for k, v in static_values.items():
                    row[k] = v
            # semantic
            for exec in executions_to_do.keys():
                file = reading[exec]
                values = extract_semantic_metrics(file)
                # write them
                for key, value in semantic_metrics.items():
                    row["{} {}".format(key, exec)] = values[value]
        except FileNotFoundError:
            pass
        writer.writerow(row)


if __name__ == "__main__":
    read_files(output_files_names)
