import argparse
import multiprocessing

from functions import *
from imitator import *
from executions import *

parser = argparse.ArgumentParser(description='')
parser.add_argument("-t", "--timeout", help='timeout for a run of IMITATOR (default: 120)',
                    default=120)
args = parser.parse_args()

number_cpus = multiprocessing.cpu_count()
number_parralel_executions = number_cpus//5  # leave more than one core free

# Locations
prop_list = "reachability_props.csv"
execution_file = os.path.join(result_directory, "executions.txt")
output_files_names = os.path.join(result_directory, "output_files.csv")
# Parameters for runs
exec_timeout = args.timeout  # in sec.

benchmarks_location = "~/benchmarks"
res_location = os.path.join(result_directory, "res")

os.system("mkdir -p {}".format(res_location))


def write_executions(property_row, execution="", timeout=exec_timeout):
    """
    Execute imitator and return the path to the res
    :param property_row:
    :param execution: key of execution_to_do
    :param timeout:
    :return:
    """
    try:
        extra = executions_to_do[execution]
    except KeyError:
        extra = ""

    model_path = ImitatorNaming.absolute_path_of_model(property_row["Path"], property_row["Model"])
    prop_path = ImitatorNaming.absolute_path_of_prop(property_row["Path"], property_row["Property"])
    extra_command = "{}{}".format(" " + extra if extra != "" else "", property_row["Extra-command"])

    res_file_path_and_name = os.path.join(res_location,
                                          def_res(property_row["Model"], property_row["Property"], timeout, execution))
    res_file_path = res_file_path_and_name + ImitatorParams.res_extension

    try:
        f = open(res_file_path, "r")
    except FileNotFoundError:
        cmd = "{} {} {}{} -output-prefix {}".format(imitator_cmd, model_path, prop_path, extra_command,
                                                    res_file_path_and_name)

        cleaning_first = "sed -i 's#{}##g' {}".format(benchmarks_location, res_file_path)
        cleaning_second = "sed -i 's#{}#{}#g' {}".format(imitator_cmd, "imitator", res_file_path)

        f = open(execution_file, "a")

        timeoutCmd = "timeout {} ".format(timeout) if timeout != 0 else ""
        f.write(timeoutCmd + " " + cmd + " > /dev/null" + " ; " + cleaning_first + " ; " + cleaning_second + "\n")
    return res_file_path


def deal(list_of_executions):
    # remove content from executions
    f = open(execution_file, "w")
    f.close()

    # write file of executions and output output names
    f = open(output_files_names, "w")
    fieldnames = ["Model", "Property"]
    for exec in executions_to_do.keys():
        fieldnames.append(exec)
    writer = csv.DictWriter(f, fieldnames=fieldnames, delimiter=csv_sep)
    writer.writeheader()
    writing = {}

    for property_row in list_of_executions:
        # write runs
        writing["Model"] = property_row["Model"]
        writing["Property"] = property_row["Property"]
        for exec, extra in executions_to_do.items():
            writing[exec] = write_executions(property_row, execution=exec)
        # write output outputnames
        writer.writerow(writing)

    # execute them
    os.system("parallel -j " + str(number_parralel_executions) + " {} < " + execution_file)


def write_readme(nb_executions):
    f = open(os.path.join(result_directory, "README.md"), "w")
    L = []
    L.append("# {}".format(experiment_name))
    L.append("- Executions:")
    L.append("- # executions: {}".format(nb_executions))
    L.append("```{}```".format(executions_to_do))
    L.append("- [scripts](scripts/)")
    L.append("- [results](results/)")
    L = [k + "\n" for k in L]
    f.writelines(L)


## main
if __name__ == "__main__":
    executions = extract_list_of_executions(prop_list)
    write_readme(len(executions))
    deal(executions)
