import argparse
import multiprocessing
import numpy

from functions import *
from imitator import *

parser = argparse.ArgumentParser(description='')
parser.add_argument("-f", "--file", help='Path to result file')
parser.add_argument("-to", "--to", help='Option for considering TOs (ignore|max)', default="ignore")
args = parser.parse_args()
args.file = "/home/dylan/workImitator/merging-experiments/results/2021-07-14_13-20-52/results.csv"  # TODO remove, for immediate test
args.to = "max"

metrics = {
    "Time": min,
    "Merging time ": min
}

# try to open file
if __name__ == '__main__':
    try:
        open(args.file, "r")
    except FileNotFoundError:
        print("File {} not found".format(args.file))


def clean_metric(metric):
    metric = metric.replace("Time ", "")
    metric = metric.replace("Number of merges ", "")
    metric = metric.replace("Merging time ", "")
    metric = metric.replace("Reconstruction time ", "")
    return metric


def value_to_float(value):
    value = value.replace(",", ".")
    return float(value) if value != "" else ""


def parseFile(result_file):
    f = open(result_file, "r")
    csv_reader = csv.DictReader(f, delimiter=',')
    results = {}
    for line in csv_reader:
        key = (line["Model"], line["Property"])
        # create results
        results[key] = {}
        for metric in metrics.keys():
            results[key][metric] = {}
        # add values
        for metric, value in line.items():
            for metric_to_keep in metrics.keys():
                if metric_to_keep in metric:
                    results[key][metric_to_keep][clean_metric(metric)] = value_to_float(value)
    return results


def get_winners(keys, values, function):
    winners = []
    winner_value = function(values)
    for k in range(len(values)):
        if values[k] == winner_value:
            winners.append(keys[k])
    return winners


def make_analysis(result_dict):
    analysis = {}
    columns = ["min", "average", "max", "winners"]
    for key, value in result_dict.items():
        for metric_to_keep in metrics:
            metric_values = result_dict[key][metric_to_keep]
            L = list(metric_values.values())
            perform_bol = True
            if "" in L:
                if args.to == "ignore":
                    print("TO for {}, ignore it".format(key))
                    perform_bol = False
                elif L == ['' for k in range(len(L))]: # check if it is not a full TO
                    perform_bol = False
                elif args.to == "max":
                    M = []
                    while "" in L: L.remove("")
                    for k in L:
                        M.append(max(L) if k == "" else k)
                    L = M
            if perform_bol:
                try:
                    analysis[key]
                except KeyError:
                    analysis[key] = {}
                analysis[key]["{} min".format(metric_to_keep)] = min(L)
                analysis[key]["{} average".format(metric_to_keep)] = numpy.mean(L)
                analysis[key]["{} max".format(metric_to_keep)] = max(L)
                analysis[key]["{} winners".format(metric_to_keep)] = get_winners(list(metric_values.keys()), L,
                                                                             metrics[metric_to_keep])
    return analysis


def count_wins(analysis_dict):
    for metric_to_keep in metrics:
        score = {}
        for key in analysis_dict.keys():
            winners = analysis_dict[key]["{} winners".format(metric_to_keep)]
            for w in winners:
                try:
                    score[w] += 1
                except KeyError:
                    score[w] = 1
        sort_score = dict(sorted(score.items(), key=lambda x: -x[1]))
        print("* Winners for {}".format(metric_to_keep))
        for k,v in sort_score.items():
            print("\t-{}: {}".format(k,v))


if __name__ == "__main__":
    results = parseFile(args.file)
    analysis = make_analysis(results)
    count_wins(analysis)
