import csv
import statistics
import os

# WARNING: we suppose (for the replaced number of states and set if a merge is done) that all the (model, property) ends at least one time for each run

# File parameters
number_runs = 5
prefix_files = 'input/results_'
suffix_files = ".csv"
delimiter = ","

# Experiment and analysis parameters
default_keep = "Number of states"
default_ignore_keep = []  # to force column check if needed
value_for_TO = 120
heuristic_none = "no-"
output_has_merges = "output_result_has_merges.csv"
output_file = "output_result_states.csv"
no_merge_execution_where_number_states_are_replaced = []


### Charge data
def charge_values(file, keep=default_keep, ignore_keep=default_ignore_keep):
    """
    Create dict from a result csv file
    :param file: result csv file
    :return: Dict { "model" : { "property" : { 'RVM': .., 'ROM' : ... } } }
    """
    dict = {}
    with open(file, "r") as csv_file:
        csv_reader = csv.DictReader(csv_file, delimiter=delimiter)
        for lines in csv_reader:
            model = lines["Model"]
            property = lines["Property"]

            try:
                dict[model][property] = {}
            except KeyError:
                dict[model] = {}
                dict[model][property] = {}

            number_max = 0
            for key, value in lines.items():
                if key in ["Model", "Property"]:
                    pass
                if keep in key:
                    bol = True
                    for k in ignore_keep:
                        if k in key:
                            bol = False
                    if bol:
                        new_key = key.replace(keep, "").replace(" ", "").replace("merge", "")
                        #if new_key in ["no-", "212", "RVMr", "OQM"]:
                        new_value = int(value) if value != "" else ""
                        number_max = max(number_max, new_value) if new_value != "" else number_max
                        dict[model][property][new_key] = new_value
            for heuristic in list(dict[model][property].keys()):
                if dict[model][property][heuristic] == "":
                    dict[model][property][
                        heuristic] = number_max  # if there is no number of states, set it to the max number
                    if heuristic == heuristic_none:
                        no_merge_execution_where_number_states_are_replaced.append([model, property])

    return dict


def charge_all(keep=default_keep, ignore_keep=default_ignore_keep):
    """
    Create dict { run0 : {results}, run2: {results}, ...}
    :return:
    """
    dict = {}
    for run in range(number_runs):
        index_run = run + 1
        dict["run{}".format(run)] = charge_values("{}{}{}".format(prefix_files, index_run, suffix_files), keep,
                                                  ignore_keep)
    return dict


### There are merges?
def write_has_merges(dict_experiment):
    """

    :param dict_experiment: output of charge all
    :return: write a csv file saying if there are merges in an execution
    """
    header = ["Model", "Property", "Has merges?"]
    dict = {}
    with open(output_has_merges, 'w') as f:
        dict_writer = csv.DictWriter(f, fieldnames=header)
        dict_writer.writeheader()

        for model, t_value in dict_experiment["run0"].items():
            for property, tt_value in t_value.items():
                row = {}
                row["Model"] = model
                row["Property"] = property

                # there is merges if there is two possible sizes of statespace, depending on heuristic
                bol = False
                # for run in dict_experiment.keys():
                if True:
                    run = "run0"
                    states_no_merge = dict_experiment[run][model][property][heuristic_none]
                    for heuristic in dict_experiment[run][model][property]:
                        if dict_experiment[run][model][property][heuristic] != states_no_merge:
                            bol = True
                            break
                # but, it can also be the case if the number of no_merge states has been replaced
                if (not bol) and [model, property] in no_merge_execution_where_number_states_are_replaced:
                    bol = True
                row["Has merges?"] = str(bol)
                dict_writer.writerow(row)


def charge_has_merges():
    """
    Charge if a (model, property) performs a merge from written file
    :return: dict { model { property : True/False } }
    """
    dict = {}
    with open(output_has_merges, "r") as csv_file:
        csv_reader = csv.DictReader(csv_file, delimiter=delimiter)
        for lines in csv_reader:
            model = lines["Model"]
            property = lines["Property"]
            has_merges = lines["Has merges?"]

            if has_merges=="True":
                bool = True
            elif has_merges=="False":
                bool=False
            else:
                raise "Invalid string"

            try:
                dict[model][property] = bool
            except KeyError:
                dict[model] = {}
                dict[model][property] = bool
    return dict


### Average, etc.
def average_for_execution(dict_experiment):
    """
    Return a dict where the result for each (model, property) is shown
    :param dict_experiment: Dict of all runs {run0: ..., run2: ...}
    :return: Dict of results {model : { property : { RVM : { avg(run1, run2, .. } } }
    """
    dict = {}
    for model, t_value in dict_experiment["run0"].items():
        dict[model] = {}
        for property, tt_value in t_value.items():
            dict[model][property] = {}
            for heuristic in tt_value.keys():
                L = []
                for key in dict_experiment.keys():
                    L.append(dict_experiment[key][model][property][heuristic])
                dict[model][property][heuristic] = statistics.mean(L)
    return dict


def results(average_dict, has_merges):
    """
    Return results (average, norm. average, norm. average with/without merges) from average_dict
    :param average_dict: result of average_for_execution
    :return: { heuristic: { average: .., norm. average : ..., ...  } }
    """
    dict = {}
    first_model = list(average_dict.keys())[0]
    first_property = list(average_dict[first_model].keys())[0]
    for heuristic in average_dict[first_model][first_property].keys():
        wins = 0
        L_average = []
        L_norm_average = []
        L_average_merges = []
        L_average_no_merges = []
        # average
        for model, t_values in average_dict.items():
            for property, tt_values in t_values.items():
                # wins
                values_for_all_heuristics = average_dict[model][property].values()
                value_for_heuristic = average_dict[model][property][heuristic]
                # win if value is the smaller but not the bigger
                if value_for_heuristic == min(values_for_all_heuristics) and value_for_heuristic != max(
                        values_for_all_heuristics):
                    wins += 1

                # averages etc.
                L_average.append(average_dict[model][property][heuristic])
                L_norm_average.append(
                    average_dict[model][property][heuristic] / average_dict[model][property][heuristic_none])
                if has_merges[model][property]:
                    L_average_merges.append(average_dict[model][property][heuristic])
                else:
                    L_average_no_merges.append(average_dict[model][property][heuristic])
        dict[heuristic] = {}
        dict[heuristic]["wins"] = wins
        dict[heuristic]["average"] = statistics.mean(L_average)
        dict[heuristic]["average with merge"] = statistics.mean(L_average_merges)
        dict[heuristic]["average without merge"] = statistics.mean(L_average_no_merges)
        dict[heuristic]["median"] = statistics.median(L_average)
        dict[heuristic]["norm average"] = statistics.mean(L_norm_average)
        dict[heuristic]["norm median"] = statistics.median(L_norm_average)
    return dict


def write_results(results):
    """
    Write a csv with results from output of results(..)
    :param results:
    :return: None
    """
    header = ["Statistic"] + list(results.keys())
    print(header)
    with open(output_file, 'w') as f:
        dict_writer = csv.DictWriter(f, fieldnames=header)
        dict_writer.writeheader()
        for stat in list(results[heuristic_none].keys()):
            row = {}
            row["Statistic"] = stat
            for heuristic in list(results.keys()):
                row[heuristic] = str(results[heuristic][stat]).replace(".",",")
            dict_writer.writerow(row)


if __name__ == "__main__":
    dict = charge_all()
    write_has_merges(dict)
    avg = average_for_execution(dict)
    has_merges = charge_has_merges()
    res = results(avg, has_merges)
    print(res)
    write_results(res)
